@@ -18,8 +18,15 @@ MISSING_PYTEST_MARKER = (
     "Test contains a call to the @idempotent decorated function: '{}',\n"
     "but the test does not have the @pytest.mark.idempotent marker. "
     "Please add this marker to your test function or test class.\n"
-    "To skip idempotency testing, add the marker with enabled=False: "
-    "@pytest.mark.idempotent(enabled=False)"
+    "- To skip idempotency checks as a one-off, add this marker to your test(s):\n"
+    "    @pytest.mark.idempotent(enabled=False)\n\n"
+    "- To stop enforcing test markers for this @idempotent function entirely, "
+    "add enforce_tests=False:\n"
+    "    @idempotent(enforce_tests=False)\n\n"
+    "- To disable this enforcing check globally, add the following function "
+    "to your conftest.py: \n"
+    "    def pytest_idempotent_enforce_tests() -> bool:\n"
+    "        return False"
 )
 IDEMPOTENCY_TEST_OUT_OF_ORDER = (
     "Idempotency tests are not in the correct sorted order.\n"
@@ -35,6 +42,14 @@ MISSING_IDEMPOTENT_FUNCTION = (
     "an @idempotent decorated function.\nEither remove the marker or use "
     "@pytest.mark.idempotent(enabled=False)."
 )
+RETURN_VALUES_NOT_EQUAL = (
+    "Return values of idempotent functions must be equal: {} != {}"
+)
+FAILED_TO_RAISE_IDEMPOTENCY_EXCEPTION = (
+    "@idempotent decorator has raises_exception={} but "
+    "the second run did not trigger the expected Exception."
+)
+
 
 # ------------------- Exceptions -------------------
 
@@ -46,14 +61,13 @@ class MissingPytestIdempotentMarker(BaseException):
     Ideally, we could use pytest.fail(), but that hangs in tests with multiple threads.
     """
 
-    message = (
-        "All tests containing @idempotent decorated functions "
-        "must use the @pytest.mark.idempotent marker."
-    )
-
 
 class ReturnValuesNotEqual(Exception):
-    message = "Return values of idempotent functions must be equal."
+    """Shows the user the differing return values of their idempotent function."""
+
+
+class FailedToRaiseIdempotencyException(Exception):
+    """Idempotent function did not raise an exception on the second run."""
 
 
 # ------------------- GlobalState -------------------
@@ -64,14 +78,14 @@ class GlobalState:
     Store essential metadata needed during the test runs.
 
     - should_run_twice: used to toggle the idempotency patch on/off.
-    - current_test_item: a reference to the current pytest test context.
+    - current_test: a reference to the current pytest test context.
     - contains_idempotent_function: True if an @idempotent decorated function called.
     - all_test_runs: dict mapping item.nodeid to bool(NO_IDEMPOTENCY_ID test passed
         and test contained at least 1 @idempotent decorated function)
     """
 
     should_run_twice: bool = False
-    current_test_item: Function | None = None
+    current_test: Function | None = None
     contains_idempotent_function: bool = True  # default True until test begins
     all_test_runs: dict[str, bool] = {}
 
@@ -89,23 +103,35 @@ def idempotent(func: _F | None) -> _F:
 
 @overload
 def idempotent(
-    *, equal_return: bool = False, enforce_tests: bool = True
+    *,
+    equal_return: bool = False,
+    raises_exception: type[Exception] | None = None,
+    enforce_tests: bool | None = None,
 ) -> Callable[[_F], _F]:
     ...  # pragma: no cover
 
 
 def idempotent(
-    func: _F | None = None, equal_return: bool = False, enforce_tests: bool = True
+    func: _F | None = None,
+    equal_return: bool = False,
+    raises_exception: type[Exception] | None = None,
+    enforce_tests: bool | None = None,
 ) -> Any:  # pragma: no cover
     """
-    No-op during runtime.
-    This marker allows Pytest to override the decorated function during
-    test-time to verify the function is idempotent (e.g. no side effects).
+    No-op during runtime. This marker allows Pytest to override the decorated function
+    during test-time to verify the function is idempotent (e.g. no side effects).
 
     Use `equal_return=True` to specify that the function should always returns
     the same output when run multiple times.
+
+    Use `raises_exception=MyException` to specify that the function will raise a
+    specific MyException in the event of an idempotency error.
+
+    Use `enforce_tests=True` to override the global config or to ensure all tests with
+    this function called use @pytest.mark.idempotent. Use `enforce_tests=False` to
+    disable this feature.
     """
-    del equal_return, enforce_tests
+    del equal_return, raises_exception, enforce_tests
 
     @wraps(cast(_F, func))
     def _idempotent_inner(user_func: _F) -> _F:
@@ -153,6 +179,7 @@ def pytest_collection(session: pytest.Session) -> None:
     def _idempotent(
         func: _F | None = None,
         equal_return: bool = False,
+        raises_exception: type[Exception] | None = None,
         enforce_tests: bool | None = None,
     ) -> Any:
         """
@@ -180,11 +207,8 @@ def pytest_collection(session: pytest.Session) -> None:
                     is acceptably idempotent, unless equal_return = True.
                 """
                 _global_state.contains_idempotent_function = True
-                assert _global_state.current_test_item is not None
-                if (
-                    _global_state.current_test_item.get_closest_marker("idempotent")
-                    is None
-                ):
+                assert _global_state.current_test is not None
+                if _global_state.current_test.get_closest_marker("idempotent") is None:
                     message = MISSING_PYTEST_MARKER.format(user_func.__qualname__)
                     if enforce_tests is None:
                         if enforce_test_setting is None or enforce_test_setting:
@@ -195,9 +219,24 @@ def pytest_collection(session: pytest.Session) -> None:
 
                 run_1 = user_func(*args, **kwargs)
                 if _global_state.should_run_twice:
-                    run_2 = user_func(*args, **kwargs)
+                    try:
+                        run_2 = user_func(*args, **kwargs)
+                    except Exception as exc:
+                        if raises_exception is not None and isinstance(
+                            exc, raises_exception
+                        ):
+                            return run_1
+                        raise
                     if equal_return and run_1 != run_2:
-                        raise ReturnValuesNotEqual(run_1, run_2)
+                        raise ReturnValuesNotEqual(
+                            RETURN_VALUES_NOT_EQUAL.format(run_1, run_2)
+                        )
+                    if raises_exception is not None:
+                        raise FailedToRaiseIdempotencyException(
+                            FAILED_TO_RAISE_IDEMPOTENCY_EXCEPTION.format(
+                                raises_exception.__qualname__
+                            )
+                        )
                 return run_1
 
             return cast(_F, run_twice)
@@ -241,7 +280,7 @@ def pytest_runtest_call(item: Function) -> None:
         elif not first_run_result:
             pytest.skip(SKIPPING_IDEMPOTENCY_CHECK)
 
-    _global_state.current_test_item = item
+    _global_state.current_test = item
     _global_state.contains_idempotent_function = False
 
 
